==> juvee_multiselect-0.0.6.tar.gz: lib/bottom_sheet/multi_select_bottom_sheet.dart <==
import 'package:flutter/material.dart';
import '../util/multi_select_item.dart';
import '../util/multi_select_actions.dart';
import '../util/multi_select_list_type.dart';

/// A bottom sheet widget containing either a classic checkbox style list, or a chip style list.
class MultiSelectBottomSheet<T> extends StatefulWidget
    with MultiSelectActions<T> {
  /// List of items to select from.
  final List<MultiSelectItem<T>> items;

  /// The list of selected values before interaction.
  final List<T> initialValue;

  /// The text at the top of the BottomSheet.
  final Widget? title;

  /// Fires when the an item is selected / unselected.
  final void Function(dynamic)? onSelectionChanged;

  /// Toggles search functionality.
  final bool searchable;

  /// Text on the confirm button.
  final Text? confirmText;

  final List selectedList;

  /// Text on the cancel button.
  final Text? cancelText;

  /// An enum that determines which type of list to render.
  final MultiSelectListType? listType;

  /// Sets the color of the checkbox or chip when it's selected.
  final Color? selectedColor;

  /// Set the initial height of the BottomSheet.
  final double? initialChildSize;

  /// Set the minimum height threshold of the BottomSheet before it closes.
  final double? minChildSize;

  /// Set the maximum height of the BottomSheet.
  final double? maxChildSize;

  /// Set the placeholder text of the search field.
  final String? searchHint;

  /// A function that sets the color of selected items based on their value.
  /// It will either set the chip color, or the checkbox color depending on the list type.
  final Color? Function(T)? colorator;

  /// Color of the chip body or checkbox border while not selected.
  final Color? unselectedColor;

  /// Icon button that shows the search field.
  final Icon? searchIcon;

  /// Icon button that hides the search field
  final Icon? closeSearchIcon;

  /// Style the text on the chips or list tiles.
  final TextStyle? itemsTextStyle;

  /// Style the text on the selected chips or list tiles.
  final TextStyle? selectedItemsTextStyle;

  /// Style the search text.
  final TextStyle? searchTextStyle;

  /// Style the search hint.
  final TextStyle? searchHintStyle;

  /// Moves the selected items to the top of the list.
  final bool separateSelectedItems;

  /// Set the color of the check in the checkbox
  final Color? checkColor;

  final bool enableSingleSelection;

  final bool enableTag;

  MultiSelectBottomSheet({
    Key? key,
    required this.items,
    required this.selectedList,
    required this.initialValue,
    this.title,
    this.onSelectionChanged,
    required this.enableSingleSelection,
    this.listType,
    this.cancelText,
    required this.enableTag,
    this.confirmText,
    this.searchable = false,
    this.selectedColor,
    this.initialChildSize,
    this.minChildSize,
    this.maxChildSize,
    this.colorator,
    this.unselectedColor,
    this.searchIcon,
    this.closeSearchIcon,
    this.itemsTextStyle,
    this.searchTextStyle,
    this.searchHint,
    this.searchHintStyle,
    this.selectedItemsTextStyle,
    this.separateSelectedItems = false,
    this.checkColor,
  }) : super(key: key);

  @override
  _MultiSelectBottomSheetState<T> createState() =>
      // ignore: no_logic_in_create_state
      _MultiSelectBottomSheetState<T>(items);
}

class _MultiSelectBottomSheetState<T> extends State<MultiSelectBottomSheet<T>> {
  final List<T> _selectedValues = [];
  bool _showSearch = false;
  List<MultiSelectItem<T>> _items;

  _MultiSelectBottomSheetState(this._items);

  @override
  void initState() {
    super.initState();
    _selectedValues.addAll(widget.initialValue);

    if (widget.enableSingleSelection == false) {
      for (int i = 0; i < _items.length; i++) {
        if (_selectedValues.contains(_items[i].id)) {
          if (_items[i].selected == false) {
            widget.selectedList
                .add({"id": _items[i].id, "value": _items[i].label});
          }
          _items[i].selected = true;
        }
      }
    }

    if (widget.separateSelectedItems) {
      _items = widget.separateSelected(_items);
    }
  }

  /// Returns a CheckboxListTile
  Widget _buildListItem(
      {required MultiSelectItem<T> item,
      required int index,
      required int length}) {
    return Theme(
      data: ThemeData(
        unselectedWidgetColor: widget.unselectedColor ?? Colors.black54,
      ),
      child: CheckboxListTile(
        checkColor: widget.checkColor,
        value: item.selected,
        activeColor: widget.colorator != null
            ? widget.colorator!(item.value) ?? widget.selectedColor
            : widget.selectedColor,
        title: Text(
          item.label,
          style: item.selected
              ? widget.selectedItemsTextStyle
              : widget.itemsTextStyle,
        ),
        controlAffinity: ListTileControlAffinity.leading,
        onChanged: (checked) {
          if (widget.enableSingleSelection) {
            setState(() {
              // _selectedValues = widget.onItemCheckedChange(
              //     _selectedValues, item.value, checked!);
              for (var element in _items) {
                element.selected = false;
              }
              _items[index].selected = checked!;

              if (checked) {
                selected = {
                  "value": _items[index].label,
                  "id": _items[index].id
                };
              } else {
                selected = null;
              }

              if (widget.onSelectionChanged != null) {
                widget.onSelectionChanged!(selected);
                // if (widget.onSelectionChanged != null) {

              }

              Navigator.of(context).pop();
            });
            // if (widget.onSelectionChanged != null) {
            //   widget.onSelectionChanged!(_items[index].label);
            // }
          } else {
            if (checked!) {
              item.selected = true;
              widget.selectedList.add({"id": item.id, "value": item.label});
            } else {
              item.selected = false;
              final index = widget.selectedList
                  .indexWhere((element) => element["id"] == item.id);

              if (index >= 0) {
                widget.selectedList.removeAt(index);
              }
            }

            setState(() {});
          }
        },
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      backgroundColor: Colors.white,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(Radius.circular(10.0))),
      title: widget.searchable == false
          ? widget.title ?? const Text("Select")
          : Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                _showSearch
                    ? Expanded(
                        child: Container(
                          padding: const EdgeInsets.only(left: 10),
                          child: TextField(
                            style: widget.searchTextStyle,
                            decoration: InputDecoration(
                              hintStyle: widget.searchHintStyle,
                              hintText: widget.searchHint ?? "Search",
                              focusedBorder: UnderlineInputBorder(
                                borderSide: BorderSide(
                                  color: widget.selectedColor ??
                                      Theme.of(context).primaryColor,
                                ),
                              ),
                            ),
                            onChanged: (val) {
                              List<MultiSelectItem<T>> filteredList = [];
                              filteredList = widget.updateSearchQuery(
                                  val,
                                  widget.items,
                                  widget.enableSingleSelection,
                                  widget.enableTag);
                              setState(() {
                                if (widget.separateSelectedItems) {
                                  _items =
                                      widget.separateSelected(filteredList);
                                } else {
                                  _items = filteredList;
                                }
                              });
                            },
                          ),
                        ),
                      )
                    : widget.title ?? const Text("Select"),
                IconButton(
                  icon: _showSearch
                      ? widget.closeSearchIcon ?? const Icon(Icons.close)
                      : widget.searchIcon ?? const Icon(Icons.search),
                  onPressed: () {
                    setState(() {
                      _showSearch = !_showSearch;
                      if (!_showSearch) {
                        if (widget.separateSelectedItems) {
                          _items = widget.separateSelected(widget.items);
                        } else {
                          _items = widget.items;
                        }
                      }
                    });
                  },
                ),
              ],
            ),
      contentPadding:
          widget.listType == null || widget.listType == MultiSelectListType.LIST
              ? const EdgeInsets.only(top: 12.0)
              : const EdgeInsets.all(20),
      content: Container(
          height: MediaQuery.of(context).size.height / 2.5,
          width: MediaQuery.of(context).size.width * 0.73,
          child: ListView.builder(
            itemCount: _items.length,
            physics: const ClampingScrollPhysics(),
            itemBuilder: (context, index) {
              return _buildListItem(
                  item: _items[index], length: _items.length, index: index);
            },
          )),
      actions: <Widget>[
        TextButton(
          child: widget.cancelText ??
              Text(
                "CANCEL",
                style: TextStyle(
                  color: (widget.selectedColor != null &&
                          widget.selectedColor != Colors.transparent)
                      ? widget.selectedColor!.withOpacity(1)
                      : Theme.of(context).primaryColor,
                ),
              ),
          onPressed: () {
            widget.onCancelTap(context, widget.initialValue);
          },
        ),
        TextButton(
          child: widget.confirmText ??
              Text(
                'OK',
                style: TextStyle(
                  color: (widget.selectedColor != null &&
                          widget.selectedColor != Colors.transparent)
                      ? widget.selectedColor!.withOpacity(1)
                      : Theme.of(context).primaryColor,
                ),
              ),
          onPressed: () {
            Navigator.of(context).pop();
            if (widget.onSelectionChanged != null) {
              // if (widget.onSelectionChanged != null) {
              if (widget.enableSingleSelection) {
                widget.onSelectionChanged!(selected);
              } else {
                widget.onSelectionChanged!(widget.selectedList);
              }
            }
          },
        )
      ],
    );
  }
}

dynamic selected = null;
List select = [];

==> juvee_multiselect-0.0.10.tar.gz: lib/bottom_sheet/multi_select_bottom_sheet.dart <==
import 'dart:developer';

import 'package:flutter/material.dart';
import '../util/multi_select_item.dart';
import '../util/multi_select_actions.dart';
import '../util/multi_select_list_type.dart';

/// A bottom sheet widget containing either a classic checkbox style list, or a chip style list.
class MultiSelectBottomSheet<T> extends StatefulWidget
    with MultiSelectActions<T> {
  /// List of items to select from.
  final List<MultiSelectItem<T>> items;

  /// The list of selected values before interaction.
  final List<T> initialValue;

  /// The text at the top of the BottomSheet.
  final Widget? title;

  /// Fires when the an item is selected / unselected.
  final void Function(dynamic)? onSelectionChanged;

  /// Toggles search functionality.
  final bool searchable;

  /// Text on the confirm button.
  final Text? confirmText;

  final List selectedList;

  /// Text on the cancel button.
  final Text? cancelText;

  /// An enum that determines which type of list to render.
  final MultiSelectListType? listType;

  /// Sets the color of the checkbox or chip when it's selected.
  final Color? selectedColor;

  /// Set the initial height of the BottomSheet.
  final double? initialChildSize;

  /// Set the minimum height threshold of the BottomSheet before it closes.
  final double? minChildSize;

  /// Set the maximum height of the BottomSheet.
  final double? maxChildSize;

  /// Set the placeholder text of the search field.
  final String? searchHint;

  /// A function that sets the color of selected items based on their value.
  /// It will either set the chip color, or the checkbox color depending on the list type.
  final Color? Function(T)? colorator;

  /// Color of the chip body or checkbox border while not selected.
  final Color? unselectedColor;

  /// Icon button that shows the search field.
  final Icon? searchIcon;

  /// Icon button that hides the search field
  final Icon? closeSearchIcon;

  /// Style the text on the chips or list tiles.
  final TextStyle? itemsTextStyle;

  /// Style the text on the selected chips or list tiles.
  final TextStyle? selectedItemsTextStyle;

  /// Style the search text.
  final TextStyle? searchTextStyle;

  /// Style the search hint.
  final TextStyle? searchHintStyle;

  /// Moves the selected items to the top of the list.
  final bool separateSelectedItems;

  /// Set the color of the check in the checkbox
  final Color? checkColor;

  final bool enableSingleSelection;

  final bool enableTag;

  MultiSelectBottomSheet({
    Key? key,
    required this.items,
    required this.selectedList,
    required this.initialValue,
    this.title,
    this.onSelectionChanged,
    required this.enableSingleSelection,
    this.listType,
    this.cancelText,
    required this.enableTag,
    this.confirmText,
    this.searchable = false,
    this.selectedColor,
    this.initialChildSize,
    this.minChildSize,
    this.maxChildSize,
    this.colorator,
    this.unselectedColor,
    this.searchIcon,
    this.closeSearchIcon,
    this.itemsTextStyle,
    this.searchTextStyle,
    this.searchHint,
    this.searchHintStyle,
    this.selectedItemsTextStyle,
    this.separateSelectedItems = false,
    this.checkColor,
  }) : super(key: key);

  @override
  _MultiSelectBottomSheetState<T> createState() =>
      // ignore: no_logic_in_create_state
      _MultiSelectBottomSheetState<T>(items);
}

class _MultiSelectBottomSheetState<T> extends State<MultiSelectBottomSheet<T>> {
  final List<T> _selectedValues = [];
  bool _showSearch = false;
  List<MultiSelectItem<T>> _items;

  _MultiSelectBottomSheetState(this._items);

  @override
  void initState() {
    super.initState();
    
    if (widget.initialValue.isNotEmpty) {
      _selectedValues.addAll(widget.initialValue);
      
    }

 

    if (widget.enableSingleSelection == false) {
      for (int i = 0; i < _items.length; i++) {
         
        if (_selectedValues.toString().contains(_items[i].id.toString())) {
          
          if (_items[i].selected == false) {
            widget.selectedList
                .add({"id": _items[i].id, "value": _items[i].label});
          }
          _items[i].selected = true;
        }
      }
    }else{
        if (selected==null) {for (int i = 0; i < _items.length; i++) {
    
        if (_selectedValues[0].toString().contains(_items[i].id.toString())) {
          
          if (_items[i].selected == false) {
            selected = {
                  "value": _items[i].label,
                  "id": _items[i].id
                };
          }
          _items[i].selected = true;
        }
      }

          
        }


    }

    if (widget.separateSelectedItems) {
      _items = widget.separateSelected(_items);
    }
  }

  /// Returns a CheckboxListTile
  Widget _buildListItem(
      {required MultiSelectItem<T> item,
      required int index,
      required int length}) {
    return Theme(
      data: ThemeData(
        unselectedWidgetColor: widget.unselectedColor ?? Colors.black54,
      ),
      child: CheckboxListTile(
        checkColor: widget.checkColor,
        value: item.selected,
        activeColor: widget.colorator != null
            ? widget.colorator!(item.value) ?? widget.selectedColor
            : widget.selectedColor,
        title: Text(
          item.label,
          style: item.selected
              ? widget.selectedItemsTextStyle
              : widget.itemsTextStyle,
        ),
        controlAffinity: ListTileControlAffinity.leading,
        onChanged: (checked) {
          if (widget.enableSingleSelection) {
            setState(() {
              // _selectedValues = widget.onItemCheckedChange(
              //     _selectedValues, item.value, checked!);
              for (var element in _items) {
                element.selected = false;
              }
              _items[index].selected = checked!;

              if (checked) {
                selected = {
                  "value": _items[index].label,
                  "id": _items[index].id
                };
              } else {
                selected = null;
              }

              if (widget.onSelectionChanged != null) {
                widget.onSelectionChanged!(selected);
                // if (widget.onSelectionChanged != null) {

              }

              Navigator.of(context).pop();
            });
            // if (widget.onSelectionChanged != null) {
            //   widget.onSelectionChanged!(_items[index].label);
            // }
          } else {
            if (checked!) {
              item.selected = true;
              widget.selectedList.add({"id": item.id, "value": item.label});
            } else {
              item.selected = false;
              final index = widget.selectedList
                  .indexWhere((element) => element["id"] == item.id);

              if (index >= 0) {
                widget.selectedList.removeAt(index);
              }
            }

            setState(() {});
          }
        },
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      backgroundColor: Colors.white,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(Radius.circular(10.0))),
      title: widget.searchable == false
          ? widget.title ?? const Text("Select")
          : Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                _showSearch
                    ? Expanded(
                        child: Container(
                          padding: const EdgeInsets.only(left: 10),
                          child: TextField(
                            style: widget.searchTextStyle,
                            decoration: InputDecoration(
                              hintStyle: widget.searchHintStyle,
                              hintText: widget.searchHint ?? "Search",
                              focusedBorder: UnderlineInputBorder(
                                borderSide: BorderSide(
                                  color: widget.selectedColor ??
                                      Theme.of(context).primaryColor,
                                ),
                              ),
                            ),
                            onChanged: (val) {
                              List<MultiSelectItem<T>> filteredList = [];
                              filteredList = widget.updateSearchQuery(
                                  val,
                                  widget.items,
                                  widget.enableSingleSelection,
                                  widget.enableTag);
                              setState(() {
                                if (widget.separateSelectedItems) {
                                  _items =
                                      widget.separateSelected(filteredList);
                                } else {
                                  _items = filteredList;
                                }
                              });
                            },
                          ),
                        ),
                      )
                    : widget.title ?? const Text("Select"),
                IconButton(
                  icon: _showSearch
                      ? widget.closeSearchIcon ?? const Icon(Icons.close)
                      : widget.searchIcon ?? const Icon(Icons.search),
                  onPressed: () {
                    setState(() {
                      _showSearch = !_showSearch;
                      if (!_showSearch) {
                        if (widget.separateSelectedItems) {
                          _items = widget.separateSelected(widget.items);
                        } else {
                          _items = widget.items;
                        }
                      }
                    });
                  },
                ),
              ],
            ),
      contentPadding:
          widget.listType == null || widget.listType == MultiSelectListType.LIST
              ? const EdgeInsets.only(top: 12.0)
              : const EdgeInsets.all(20),
      content: Container(
          height: MediaQuery.of(context).size.height / 2.5,
          width: MediaQuery.of(context).size.width * 0.73,
          child: ListView.builder(
            itemCount: _items.length,
            physics: const ClampingScrollPhysics(),
            itemBuilder: (context, index) {
              return _buildListItem(
                  item: _items[index], length: _items.length, index: index);
            },
          )),
      actions: <Widget>[
        TextButton(
          child: widget.cancelText ??
              Text(
                "CANCEL",
                style: TextStyle(
                  color: (widget.selectedColor != null &&
                          widget.selectedColor != Colors.transparent)
                      ? widget.selectedColor!.withOpacity(1)
                      : Theme.of(context).primaryColor,
                ),
              ),
          onPressed: () {
            widget.onCancelTap(context, widget.initialValue);
          },
        ),
        TextButton(
          child: widget.confirmText ??
              Text(
                'OK',
                style: TextStyle(
                  color: (widget.selectedColor != null &&
                          widget.selectedColor != Colors.transparent)
                      ? widget.selectedColor!.withOpacity(1)
                      : Theme.of(context).primaryColor,
                ),
              ),
          onPressed: () {
            Navigator.of(context).pop();
            if (widget.onSelectionChanged != null) {
              // if (widget.onSelectionChanged != null) {
              if (widget.enableSingleSelection) {
                widget.onSelectionChanged!(selected);
              } else {
                widget.onSelectionChanged!(widget.selectedList);
              }
            }
          },
        )
      ],
    );
  }
}

dynamic selected = null;
List select = [];
